SystemOrganization addCategory: #'Alien-Core'!

!String methodsFor: '*Alien-Core' stamp: 'F Olivero 5/18/2009 19:00'!
asAlien
	| size alien |
	size := self byteSize.
	alien := Alien newC: size + 1.
	alien replaceFrom: 1 to: size with: self startingAt: 1.
	^ alien! !

!False methodsFor: '*Alien-Core' stamp: 'John M McIntosh 11/17/2008 17:02'!
asAlien
	^ 0! !

!Integer methodsFor: '*Alien-Core' stamp: 'eem 1/16/2008 13:57'!
| anInteger
	^self bitOr: anInteger! !

!MethodNode methodsFor: '*Alien-Core-First' stamp: 'John M McIntosh 2/23/2009 23:05'!
generate
	"The receiver is the root of a parse tree. Answer a CompiledMethod. The
	argument, trailer, is the references to the source code that is stored with 
	every CompiledMethod."

	^self generate: #(0 0 0 0)! !

!MethodNode methodsFor: '*Alien-Core-override' stamp: 'eem 7/13/2007 14:00'!
generateWith: trailer using: aCompiledMethodClass
	"The receiver is the root of a parse tree. Answer a CompiledMethod. The
	argument, trailer, is the references to the source code that is stored with 
	every CompiledMethod."

	| blkSize method nArgs nLits primErrNode stack strm |
	self generate: trailer ifQuick: 
		[:m |  method := m.
		method properties: properties.
		^ method].
	primErrNode := self primitiveErrorVariableName ifNotNil:
						[encoder fixTemp: self primitiveErrorVariableName].
	nArgs := arguments size.
	blkSize := (block sizeForEvaluatedValue: encoder)
				+ (primErrNode ifNil: [0] ifNotNil: [2 "We force store-long (129)"]).
	literals := encoder allLiterals.
	nLits := literals size.
	method := aCompiledMethodClass	"Dummy to allocate right size"
				newBytes: blkSize
				trailerBytes: trailer 
				nArgs: nArgs
				nTemps: encoder maxTemp
				nStack: 0
				nLits: nLits
				primitive: primitive.
	strm := ReadWriteStream with: method.
	strm position: method initialPC - 1.
	stack := ParseStack new init.
	primErrNode ifNotNil: [primErrNode emitStore: stack on: strm].
	block emitForEvaluatedValue: stack on: strm.
	stack position ~= 1 ifTrue: [^self error: 'Compiler stack
discrepancy'].
	strm position ~= (method size - trailer size) 
		ifTrue: [^self error: 'Compiler code size discrepancy'].
	method needsFrameSize: stack size.
	1 to: nLits do: [:lit | method literalAt: lit put: (literals at: lit)].
	method properties: properties.
	^ method! !

!MethodNode methodsFor: '*Alien-Core-First' stamp: 'eem 7/9/2007 16:49'!
primitiveErrorVariableName
	"Just answer the first.  Where should we validate that there's only one error name?"
	(primitive notNil and: [primitive > 0]) ifTrue:
		[properties pragmas do:
			[:pragma| | kws |
			kws := pragma keyword keywords.
			(kws size >= 2
			 and: [kws first = 'primitive:'
			 and: [kws second = 'error:']]) ifTrue:
				[^pragma argumentAt: 2]]].
	^nil

	"(Parser new parse: (MethodNode sourceCodeAt: #primitiveErrorVariableName) class: Parser) primitiveErrorVariableName"

	"(Parser new parse: 'foo <primitive: 111 error: ''foo''> self primitiveFailed' class: Object) primitiveErrorVariableName"

	"(Parser new parse: 'foo <primitive: 111 error: foo> self primitiveFailed' class: Object) primitiveErrorVariableName"

	"(Parser new parse: 'foo <primitive: 111> self primitiveFailed' class: Object) primitiveErrorVariableName"

	"(Parser new parse: 'foo <primitive: 111 error: foo> self primitiveFailed' class: Object) generate"! !

!MethodNode methodsFor: '*Alien-Core-override' stamp: 'eem 1/2/2008 18:11'!
printOn: aStream 
	precedence = 1
		ifTrue: 
			[aStream nextPutAll: self selector]
		ifFalse: 
			[self selector keywords with: arguments do: 
				[:kwd :arg | 
				aStream nextPutAll: kwd; space.
				aStream withStyleFor: #methodArgument
					do: [aStream nextPutAll: arg key].
				aStream space]].
	comment == nil ifFalse: 
			[aStream crtab: 1.
			self printCommentOn: aStream indent: 1].
	temporaries size > 0 ifTrue: 
			[aStream crtab: 1.
			aStream nextPutAll: '|'.
			aStream withStyleFor: #temporaryVariable
				do: [temporaries do: 
						[:temp | aStream space; nextPutAll: temp key]].
				aStream nextPutAll: ' |'].
	properties ifNotNil: [ 
		properties pragmas do: [ :each |
			"Don't decompile basic primitives that return self, i-vars, etc."
			each keyword keywords first = 'primitive:'
				ifFalse: [ aStream crtab: 1. each printOn: aStream ]
				ifTrue: [
					((each argumentAt: 1) isInteger
					 and: [(each argumentAt: 1) between: 255 and: 519])
						ifFalse: [ aStream crtab: 1. self printPrimitiveOn: aStream ] ] ] ].
	aStream crtab: 1.
	^ block printStatementsOn: aStream indent: 0! !

!MethodNode methodsFor: '*Alien-Core-override' stamp: 'eem 7/13/2007 16:50'!
printPrimitiveOn: aStream 
	"Print the primitive on aStream"
	| primIndex primDecl |
	primIndex := primitive.
	primIndex = 0 ifTrue: [^ self].
	primIndex = 120 ifTrue: "External call spec"
		[^ aStream print: encoder literals first].
	aStream nextPutAll: '<primitive: '.
	primIndex = 117
		ifTrue:
			[primDecl := encoder literals at: 1.
			 aStream nextPut: $'; nextPutAll: (primDecl at: 2); nextPut: $']
		ifFalse: [aStream print: primIndex].
	self primitiveErrorVariableName notNil ifTrue:
		[aStream nextPutAll: ' error: '; nextPutAll: self primitiveErrorVariableName].
	primIndex = 117 ifTrue:
		 [primDecl first notNil ifTrue:
			[aStream
				nextPutAll: ' module:';
				nextPut: $';
				nextPutAll: primDecl first;
				nextPut: $']].
	aStream nextPut: $>.
	Smalltalk at: #Interpreter ifPresent:[:cls|
		aStream nextPutAll: ' "'
				, ((cls classPool at: #PrimitiveTable)
						at: primIndex + 1) , '" ']! !

!MethodNode methodsFor: '*Alien-Core' stamp: 'eem 7/13/2007 17:08'!
removeAndRenameLastTempIfErrorCode
	self primitiveErrorVariableName ifNotNil:
		[temporaries last name: self primitiveErrorVariableName.
		 temporaries removeLast].! !

!CompiledMethod methodsFor: '*Alien-Core' stamp: 'eem 7/13/2007 17:13'!
primitiveErrorVariableName
	"Just answer the first.  Where should we validate that there's only one error name?"
	self properties pragmas do:
		[:pragma| | kws |
		kws := pragma keyword keywords.
		(kws size >= 2
		 and: [kws first = 'primitive:'
		 and: [kws second = 'error:']]) ifTrue:
			[^pragma argumentAt: 2]].
	^nil! !

!CompiledMethod methodsFor: '*Alien-Core-override' stamp: 'eem 7/13/2007 17:10'!
printPrimitiveOn: aStream
	"Print the primitive on aStream"
	| primIndex primDecl |
	primIndex := self primitive.
	primIndex = 0 ifTrue:[^self].
	primIndex = 120 ifTrue:"External call spec"
		[^aStream print: (self literalAt: 1); cr].
	aStream nextPutAll: '<primitive: '.
	primIndex = 117
		ifTrue:
			[primDecl := self literalAt: 1.
			 aStream nextPut: $'; nextPutAll: (primDecl at: 2); nextPut: $']
		ifFalse: [aStream print: primIndex].
	self primitiveErrorVariableName notNil ifTrue:
		[aStream nextPutAll: ' error: '; nextPutAll: self primitiveErrorVariableName].
	primIndex = 117 ifTrue:
		 [primDecl first notNil ifTrue:
			[aStream
				nextPutAll: ' module:';
				nextPut: $';
				nextPutAll: primDecl first;
				nextPut: $']].
	aStream nextPut: $>; cr! !

!WeakArray class methodsFor: '*Alien-Core' stamp: 'eem 2/20/2008 13:57'!
hasWeakDependent: anObject
	^FinalizationDependents includes: anObject! !

!Compiler class methodsFor: '*Alien-Core' stamp: 'Ahe 8/23/2007 15:20'!
defaultParserClass
	"Return a parser class to use for parsing method headers."

	^Parser! !

!Compiler class methodsFor: '*Alien-Core' stamp: 'eem 1/23/2007 17:38'!
extension
	^self == Compiler
		ifTrue: ['st']
		ifFalse: [self subclassResponsibility]! !

!Compiler class methodsFor: '*Alien-Core-override' stamp: 'Ahe 8/23/2007 15:21'!
new

	^ super new parserClass: self defaultParserClass! !

!Compiler methodsFor: '*Alien-Core' stamp: 'GB 11/8/2007 15:17'!
decompilerClass
	^Decompiler! !

!Compiler methodsFor: '*Alien-Core' stamp: 'GB 11/8/2007 15:06'!
evaluate: textOrString for: anObject notifying: aController logged: logFlag
	"Compile and execute the argument, textOrString with respect to the class 
	of anObject. If a compilation error occurs, notify aController. If both 
	compilation and execution are successful then, if logFlag is true, log 
	(write) the text onto a system changes file so that it can be replayed if 
	necessary."

	^ self 
				evaluate: textOrString
				in: nil
				to: anObject
				notifying: aController
				ifFail: [^nil]
				logged: logFlag.! !

!Compiler methodsFor: '*Alien-Core-override' stamp: 'GB 8/17/2007 13:12'!
evaluate: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock logged: logFlag
	"Compiles the sourceStream into a parse tree, then generates code into a method. In other words, if receiver is not nil, then the text can refer to instance variables of that receiver (the Inspector uses this). If aContext is not nil, the text can refer to temporaries in that context (the Debugger uses this). If aRequestor is not nil, then it will receive a notify:at: message before the attempt to evaluate is aborted. Finally, the compiled method is directly invoked without modifying the receiving-class."

	| methodNode method value toLog itsSelectionString itsSelection |
	class := (aContext isNil 
		ifTrue: [ receiver ] 
		ifFalse: [ aContext receiver ])
			class.
	self from: textOrStream class: class context: aContext notifying: aRequestor.
	methodNode := self 
		translate: sourceStream
		noPattern: true 
		ifFail: [ ^ failBlock value ].
	method := methodNode generate.
	method selector ifNil: [method selector: (aContext isNil ifTrue: [#DoIt] ifFalse: [#DoItIn:])].
	"self interactive
		ifTrue: [ method := method copyWithTempNames: methodNode tempNames ]."
	value := receiver 
		withArgs: (context isNil
			ifTrue: [ #() ]
			ifFalse: [ Array with: aContext ])
		executeMethod: method.
	logFlag ifTrue:
		[toLog := ((requestor respondsTo: #selection)  and:
			[(itsSelection := requestor selection) notNil] and:
			[(itsSelectionString := itsSelection asString) isEmptyOrNil not] )
			ifTrue: 
				[itsSelectionString]
			ifFalse:
				[sourceStream contents].
		SystemChangeNotifier uniqueInstance evaluated: toLog context: aContext ].
	^ value.! !

!Compiler methodsFor: '*Alien-Core' stamp: 'GB 11/8/2007 15:06'!
evaluate: textOrString notifying: aController logged: logFlag 
	"See Compiler|evaluate:for:notifying:logged:. Compilation is carried out 
	with respect to nil, i.e., no object."

	^self evaluate: textOrString for: nil notifying: aController logged: logFlag! !

!SequenceableCollection methodsFor: '*Alien-Core' stamp: 'kdt 10/21/2009 15:05'!
copyGrownBy: length
	| newCollection |
	newCollection := self species ofSize: self size + length.
	newCollection replaceFrom: 1 to: self size with: self startingAt: 1.
	^ newCollection! !

ByteArray variableByteSubclass: #Alien
	instanceVariableNames: ''
	classVariableNames: 'GCMallocedAliens PlatformSizes ProcessIoEvents'
	poolDictionaries: ''
	category: 'Alien-Core'!

!Alien commentStamp: 'John M McIntosh 11/23/2008 12:45' prior: 0!
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0

Instances of Alien represent actual parameters, return results and function pointers in FFI call-outs and call-backs and provide handles on external data.  See NewsqueakIA32ABIPlugin for the VM code that actually implements call-outs and call-backs.

See the class-side examples category for some simple example workspaces.

Aliens represent ABI (C language) data.  They can hold data directly in their bytes or indirectly by pointing to data on the C heap.  Alien instances are at least 5 bytes in length. The first 4 bytes of an Alien hold the size, as a signed integer, of the datum the instance is a proxy for.  If the size is positive then the Alien is "direct" and the actual datum resides in the object itself, starting at the 5th byte.  If the size is negative then the proxy is "indirect", is at least 8 bytes in length and the second 4 bytes hold the address of the datum, which is assumed to be on the C heap.  Any attempt to access data beyond the size will fail.  If the size is zero then the Alien is a pointer, the second 4 bytes hold a pointer, as for "indirect" Aliens, and accessing primitives indirect through the pointer to access data, but no bounds checking is performed.

When Aliens are used as parameters in FFI calls then all are "passed by value", so that e.g. a 4 byte direct alien will have its 4 bytes of data passed, and a 12-byte indirect alien will have the 12 bytes its address references passed.  Pointer aliens will have their 4 byte pointer passed.  So indirect and pointer aliens are equivalent for accessing data but different when passed as parameters, indirect Aliens passing the data and pointer Aliens passing the pointer.

Class Variables:
GCMallocedAliens <AlienWeakTable of <Alien -> Integer>> - weak collection of malloced aliens, used to free malloced memory of Aliens allocated with newGC:
LoadedLibraries <Dictionary of <String -> Alien>> - library name to library handle map!

!Alien class methodsFor: 'primitives-memory management' stamp: 'eem 11/8/2007 16:20'!
Ccalloc: byteSize "<Integer> ^<Integer>"
	<primitive: 'primCalloc' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien class methodsFor: 'primitives-memory management' stamp: 'eem 11/8/2007 16:20'!
Cmalloc: byteSize "<Integer> ^<Integer>"
	<primitive: 'primMalloc' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:15'!
atAddress: pointer "<Integer>" "^<Alien>"
	"N.B.  This method assumes one is creating a pointer to already initialized data, hence initiaize is not sent.
	(Perhaps some alternative, eg. initializePointer, should be sent?"
	^(self basicNew: self sizeofLong + self sizeofPointer)
		sizeFieldPut: self dataSize negated;
		addressFieldPut: pointer;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:15'!
atAddress: address "<Integer>" dataSize: dataByteSize "<Integer>" "^<Alien>"
	"N.B.  This method assumes one is creating a pointer to already initialized data, hence initiaize is not sent.
	(Perhaps some alternative, eg. initializePointer, should be sent?"
	^(self basicNew: self sizeofLong + self sizeofPointer)
		sizeFieldPut: dataByteSize asInteger abs negated;
		addressFieldPut: address;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/6/2007 11:46'!
dataSize "^<Integer>"
	"Answer the number of bytes required to represent an instace of the receiver.
	This excludes the size word's own bytes and is just the size of the data."
	^self subclassResponsibility! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:12'!
doubleSize
	^ self sizeofDouble! !

!Alien class methodsFor: 'class initialization' stamp: 'F Olivero 5/19/2009 01:32'!
ensureInSpecialObjectsArray
	"Alien must be in the specialObjectsArray to enable the Newsqueak FFI.  Because of
	 the bootstrap the specialObjectsArray may already have been partially initialized with
	 the callback selector and to be large enough (as part of loading immutability), but
	 it will be missing Alien until Alien is loaded.  So check if the specialObjectsArray is
	 of the expected size before recreating and slam Alien in if the specialObjectsArray
	 is already large enough."

	| index |
	index := self specialObjectsArrayIndex.
	((Smalltalk includesKey: #ObjectMemory)
	 and: [((Smalltalk at: #ObjectMemory) classPool at: #ClassAlien ifAbsent: []) ~~ (index - 1)]) ifTrue:
		[self error: 'index probably wrong'].

	Smalltalk specialObjectsArray size < index ifTrue:
		[Smalltalk recreateSpecialObjectsArray].
	
	(Smalltalk specialObjectsArray size < index
	 or: [(Smalltalk specialObjectsArray at: index) ~~ nil
		and: [(Smalltalk specialObjectsArray at: index) ~~ self]]) ifTrue:
		[self error: 'SystemDictionary>>recreateSpecialObjectsArray appears incompatible'].

	(Smalltalk specialObjectsArray at: index) == nil ifTrue:
		[Smalltalk specialObjectsArray at: index put: self]! !

!Alien class methodsFor: 'class initialization' stamp: 'eem 1/7/2008 19:33'!
ensureNotifiedAtStartup
	"Alien needs to be notified at system startup to clear the LoadedLibraries table
	and otherwise sanitise Aliens with dangling pointers."
	self ~~ Alien ifTrue: [^self].
	Smalltalk addToStartUpList: self after: Delay! !

!Alien class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:44'!
ensureNotifiedAtStartupAfter: anotherClass
	"Support routine for putting Alien classes in the start-up list."
	anotherClass ensureNotifiedAtStartup.
	Smalltalk addToStartUpList: self after: anotherClass! !

!Alien class methodsFor: 'finalization' stamp: 'eem 12/21/2007 15:15'!
finalize: mallocTag
	self primFree: mallocTag! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:12'!
floatSize
	^ self sizeofFloat! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:16'!
forPointer: address "<Integer>" "^<Alien>"
	"N.B.  This method assumes one is creating a pointer to already initialized data, hence initiaize is not sent.
	(Perhaps some alternative, eg. initializePointer, should be sent?"
	^(self basicNew: self sizeofLong + self sizeofPointer)
		sizeFieldPut: 0;
		addressFieldPut: address;
		yourself! !

!Alien class methodsFor: 'class initialization' stamp: 'kdt 9/25/2009 16:45'!
initialize
	"
	Alien initialize
	"
	PlatformSizes := nil.
	GCMallocedAliens isNil ifTrue:
		[GCMallocedAliens := AlienWeakTable newForOwner: self].
	self ensureInSpecialObjectsArray.
	self ensureNotifiedAtStartup! !

!Alien class methodsFor: 'callbacks' stamp: 'vb 5/9/2008 21:43'!
invokeCallback: thunkPtr "<Integer>" stack: stackPtr "<Integer>" registers: regsPtr "<Integer>" jmpbuf: jmpBufPtr "<Integer>" "^<FFICallbackReturnValue>"
	"The low-level entry-point for callbacks sent from the VM/IA32ABI plugin.
	 Return via primReturnFromContext:through:.  thisContext's sender is the
	 call-out context."
	| resultProxy |
	[resultProxy := Callback evaluateCallbackForThunk: thunkPtr stack: stackPtr]
		ifCurtailed: [self error: 'attempt to non-local return across a callback'].
	resultProxy ifNil:
		[(resultProxy := FFICallbackReturnValue new) returnInteger: 0].
	resultProxy primReturnFromContext: thisContext through: jmpBufPtr! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/27/2007 10:47'!
new
	"Answer a new instance of the default size whose data resides on the Smalltalk heap directly in the instance."
	^self new: self dataSize! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:14'!
new: byteSize
	"Answer a new instance of the given byte size whose data resides on trhe Smalltalk heap directly in the instance."
	^(self basicNew: self sizeofLong + byteSize )
		sizeFieldPut: byteSize;
		initialize! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 11/6/2007 12:13'!
newC
	"Answer a new instance of the default size whose data resides on the external C heap."
	^self newC: self dataSize! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:16'!
newC: byteSize
	"Answer a new instance of the given byte size whose data resides on the external C heap.
	 As with instances residing on the Smalltalk heap the data is zero-filled."
	^(self basicNew: self sizeofLong + byteSize)
		sizeFieldPut: byteSize negated;
		addressFieldPut: (self Ccalloc: byteSize);
		initialize! !

!Alien class methodsFor: 'instance creation' stamp: 'vb 12/6/2007 11:11'!
newCString: aString "<String> ^<Alien>"
	"Answer an Alien pointing to a null-terminated copy of aString on the external C heap."

	| size cString |
	size := aString size.
	^(Alien rawNewC: size + 1)
		replaceFrom: 1 to: size with: aString startingAt: 1;
		unsignedByteAt: size + 1 put: 0;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:17'!
newForCallbackArgs
	"Answer an Alien suitable for wrapping the stack pointer of a callback, hence
	 providing access to its args.  The actual stack pointer is slammed in in
	 Callback>>valueWithStackPointer: prior to each evaluation of the callback block"
	^(self basicNew: self sizeofLong + self sizeofPointer)
		sizeFieldPut: self dataSize negated;
		yourself! !

!Alien class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 14:13'!
newGC
	"Answer a new instance of the default size whose data resides on the external C heap,
	 and whose dsata will be reclaimed automatically when the instance is garbage collected."
	^self newGC: self dataSize! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:17'!
newGC: byteSize
	"Answer a new instance of the given byte size whose data resides on the external C heap,
	 and whose dsata will be reclaimed automatically when the instance is garbage collected.
	 As with instances residing on the Smalltalk heap the data is zero-filled."
	| alien address |
	(alien := self basicNew: self sizeofLong + self sizeofPointer)
		sizeFieldPut: byteSize negated.
	address := self Ccalloc: byteSize.
	GCMallocedAliens add: alien finalizing: address.
	^alien
		addressFieldPut: address;
		initialize! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:46'!
platformSizes
	^ PlatformSizes ifNil: [ PlatformSizes := self platformSizesIA32 ]! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/25/2009 16:43'!
platformSizesIA32
	^ IdentityDictionary new
		add: #char -> 1;
		add: #short -> 2;
		add: #int -> 4;
		add: #long -> 4;
		add: #longLong -> 8;
		add: #float -> 4;
		add: #double -> 8;
		add: #pointer -> 4;
		yourself! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/25/2009 16:44'!
platformSizesLLP64
	"MS Windows 64-bit"
	^ IdentityDictionary new
		add: #char -> 1;
		add: #short -> 2;
		add: #int -> 4;
		add: #long -> 4;
		add: #longLong -> 8;
		add: #float -> 4;
		add: #double -> 8;
		add: #pointer -> 8;
		yourself! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/25/2009 16:44'!
platformSizesLP64
	"64-bit Unix, including Mac OS X"
	^ IdentityDictionary new
		add: #char -> 1;
		add: #short -> 2;
		add: #int -> 4;
		add: #long -> 8;
		add: #longLong -> 8;
		add: #float -> 4;
		add: #double -> 8;
		add: #pointer -> 8;
		yourself! !

!Alien class methodsFor: 'primitives-memory management' stamp: 'eem 11/8/2007 16:21'!
primFree: address "<Integer>"
	<primitive: 'primFree' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien class methodsFor: 'instance creation' stamp: 'kdt 9/22/2009 12:17'!
rawNewC: byteSize
	"Answer a new instance of the given byte size whose data resides on the external C heap.
	 Unlike newC: the data is *not* zero-filled."
	^(self basicNew: self sizeofLong + self sizeofPointer)
		sizeFieldPut: byteSize negated;
		addressFieldPut: (self Cmalloc: byteSize);
		initialize! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:17'!
signedLongSize
	^ self sizeofLong! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:18'!
signedShortSize
	^ self sizeofShort! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:50'!
sizeofChar
	^self platformSizes at: #char! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:51'!
sizeofDouble
	^self platformSizes at: #double! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:51'!
sizeofFloat
	^self platformSizes at: #float! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:50'!
sizeofInt
	^self platformSizes at: #int! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:50'!
sizeofLong
	^self platformSizes at: #long! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:51'!
sizeofLongLong
	^self platformSizes at: #longLong! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:51'!
sizeofPointer
	^self platformSizes at: #pointer! !

!Alien class methodsFor: 'platform' stamp: 'kdt 9/21/2009 15:50'!
sizeofShort
	^self platformSizes at: #short! !

!Alien class methodsFor: 'class initialization' stamp: 'F Olivero 5/19/2009 01:31'!
specialObjectsArrayIndex
	^ 53! !

!Alien class methodsFor: 'system startup' stamp: 'F Olivero 5/18/2009 20:54'!
startUp: resuming
	resuming ifTrue:
		["The io lock is off in the new session, so we shut down the process if it's running."
		ProcessIoEvents := false]! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:18'!
unsignedCharSize
	^ self sizeofChar! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:18'!
unsignedLongSize
	^ self sizeofLong! !

!Alien class methodsFor: 'type sizes' stamp: 'kdt 9/22/2009 12:18'!
unsignedShortSize
	^ self sizeofShort! !

!Alien methodsFor: 'accessing' stamp: 'eem 2/5/2008 16:27'!
address
	^self sizeField <= 0
		ifTrue: [self addressField]
		ifFalse: [self error: 'One cannot take the address of direct Alien instances; they may move']! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/11/2007 12:00'!
addressField "^<Integer>"
	<primitive: 'primAddressField' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/11/2007 12:01'!
addressFieldPut: value "<Integer> ^<Integer>"
	<primitive: 'primAddressFieldPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'coercing' stamp: 'eem 1/10/2008 22:01'!
asAlien
	^self! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asSignedByte "^<Integer>"
	"Answer an Integer created from the first byte of the receiver interpreted as a signed value."
	
	^self signedByteAt: 1! !

!Alien methodsFor: 'converting' stamp: 'kdt 9/22/2009 11:36'!
asSignedInt "^<Integer>"
	"Answer an Integer created from the first <n> bytes of the receiver (where n = self class sizeofInt) interpreted as an unsigned value"
	
	^self signedIntAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asSignedLong "^<Integer>"
	"Answer an Integer created from the first 4 bytes of the receiver interpreted as a signed value."
	
	^self signedLongAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asSignedShort "^<Integer>"
	"Answer an Integer created from the first 2 bytes of the receiver interpreted as a signed value."
	
	^self signedShortAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asUnsignedByte "^<Integer>"
	"Answer an Integer created from the first byte of the receiver interpreted as an unsigned value."
	
	^self unsignedByteAt: 1! !

!Alien methodsFor: 'converting' stamp: 'kdt 9/22/2009 11:36'!
asUnsignedInt "^<Integer>"
	"Answer an Integer created from the first <n> bytes of the receiver (where n = self class sizeofInt) interpreted as an unsigned value"
	
	^self unsignedIntAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asUnsignedLong "^<Integer>"
	"Answer an Integer created from the first 4 bytes of the receiver interpreted as an unsigned value."
	
	^self unsignedLongAt: 1! !

!Alien methodsFor: 'converting' stamp: 'vb 12/6/2007 11:24'!
asUnsignedShort "^<Integer>"
	"Answer an Integer created from the first 2 bytes of the receiver interpreted as an unsigned value."
	
	^self unsignedShortAt: 1! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/29/2007 11:56'!
copyInto: aByteObject "<Alien | indexableByteSubclass>" from: start "<Integer>" to: stop "<Integer>" in: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>"
	<primitive: 'primAlienReplace' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'accessing' stamp: 'eem 12/20/2007 17:41'!
dataSize
	^self sizeField abs! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:22'!
doubleAt: index "<Integer> ^<Float>"
	<primitive: 'primDoubleAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:25'!
doubleAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>"
	<primitive: 'primDoubleAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:24'!
floatAt: index "<Integer> ^<Float>"
	<primitive: 'primFloatAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:24'!
floatAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>"
	<primitive: 'primFloatAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'memory management' stamp: 'eem 12/21/2007 15:50'!
free
	self primFree! !

!Alien methodsFor: 'accessing' stamp: 'eem 4/18/2008 15:14'!
freeAfter: aBlock
	"The receiver must be a pointer to a previously allocated external C heap memory block. Invoke aBlock passing the receiver as the argument, then free the receiver. Allows automatic allocation and freeing of C artifacts using patters such as
	(Alien newCString: 'foo') freeAfter: [:cstring | ...use cstring here...]."
	
	^[aBlock value: self] ensure: [self free]! !

!Alien methodsFor: 'instance initalization' stamp: 'eem 11/6/2007 12:09'!
initialize
	"Subclasses should redefine this method to perform initializations on instance creation"
	"Overrides ProtoObjects' method for explicitness."! !

!Alien methodsFor: 'accessing' stamp: 'eem 2/5/2008 16:27'!
pointer
	"Answer a pointer to the receiver in the form of an Alien.
	 This will fail if the receiver is not indirect."
	self sizeField = 0 ifTrue:
		[self error: 'One cannot take the address of pointer Alien instances; they may move, and are already pointers'].
	^self class forPointer: self address! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result 	"<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg 	"<Alien | Integer | Double> ^<Alien>"
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result 	"<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>"
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature:  byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>"
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>"
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result 	"<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>"
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature:  byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" with: arg9  "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" with: arg9 "<Alien | Integer | Double> ^<Alien>" with: arg10  "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "Alien" floatParmsSignature: byteArray "ByteArray" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" with: arg9 "<Alien | Integer | Double> ^<Alien>" with: arg10 "<Alien | Integer | Double> ^<Alien>" with: arg11 "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" floatParmsSignature: args "<ByteArray | nil>"  withArguments: anObject "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primVarArgsCallOutDoubleReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer> ^<Alien>" with: thirdArg "<Alien | SmallInteger> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg with: fifth "<Alien | SmallInteger> ^<Alien>""<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer> ^<Alien>" with: seventh "<Alien | SmallInteger> ^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" with: arg12 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallDoubleResult: result "<Alien>" withArguments: args "<Array of: Alien | Integer> ^<Alien>"
	<primitive: 'primVarArgsCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature:  byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg  "<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg 	"<Alien | Integer | Double> ^<Alien>" with: secondArg 	"<Alien | Integer | Double> ^<Alien>" with: thirdArg 	"<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" 

	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" 

	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" with: arg9 "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" with: arg9 "<Alien | Integer | Double> ^<Alien>" with: arg10 "<Alien | Integer | Double> ^<Alien>"  
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" with: arg7 "<Alien | Integer | Double> ^<Alien>" with: arg8 "<Alien | Integer | Double> ^<Alien>" with: arg9 "<Alien | Integer | Double> ^<Alien>" with: arg10 "<Alien | Integer | Double> ^<Alien>" with: arg11 "<Alien | Integer | Double> ^<Alien>"  
	
	<primitive: 'primCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result floatParmsSignature:  byteArray "<ByteArray | nil>"  withArguments: aCollection  "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primVarArgsCallOutFloatReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer> ^<Alien>" with: thirdArg "<Alien | SmallInteger> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg with: fifth "<Alien | SmallInteger> ^<Alien>""<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer> ^<Alien>" with: seventh "<Alien | SmallInteger> ^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8  with: arg9"<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" with: arg12 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallFloatResult: result "<Alien>" withArguments: args "<Array of: Alien | Integer> ^<Alien>"
	<primitive: 'primVarArgsCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>"  with: secondArg "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>"floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>" with: thirdArg "<Alien | Integer | Double> ^<Alien>" with: fourthArg "<Alien | Integer | Double> ^<Alien>" with: fifthArg "<Alien | Integer | Double> ^<Alien>" with: sixthArg "<Alien | Integer | Double> ^<Alien>" 
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>"with: secondArg "<Alien | Integer | Double> ^<Alien>"with: thirdArg "<Alien | Integer | Double> ^<Alien>"with: fourthArg "<Alien | Integer | Double> ^<Alien>"with: fifthArg "<Alien | Integer | Double> ^<Alien>"with: sixthArg "<Alien | Integer | Double> ^<Alien>"with: arg7 "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>" with: secondArg "<Alien | Integer | Double> ^<Alien>"with: thirdArg "<Alien | Integer | Double> ^<Alien>"with: fourthArg "<Alien | Integer | Double> ^<Alien>"with: fifthArg "<Alien | Integer | Double> ^<Alien>"with: sixthArg "<Alien | Integer | Double> ^<Alien>"with: arg7 "<Alien | Integer | Double> ^<Alien>"with: arg8 "<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>" with: firstArg "<Alien | Integer | Double> ^<Alien>"with: secondArg "<Alien | Integer | Double> ^<Alien>"with: thirdArg "<Alien | Integer | Double> ^<Alien>"with: fourthArg "<Alien | Integer | Double> ^<Alien>"with: fifthArg "<Alien | Integer | Double> ^<Alien>"with: sixthArg "<Alien | Integer | Double> ^<Alien>"with: arg7 "<Alien | Integer | Double> ^<Alien>"with: arg8 "<Alien | Integer | Double> ^<Alien>"with: arg9 "<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>"with: secondArg "<Alien | Integer | Double> ^<Alien>"with: thirdArg "<Alien | Integer | Double> ^<Alien>"with: fourthArg "<Alien | Integer | Double> ^<Alien>"with: fifthArg "<Alien | Integer | Double> ^<Alien>"with: sixthArg "<Alien | Integer | Double> ^<Alien>"with: arg7 "<Alien | Integer | Double> ^<Alien>"with: arg8 "<Alien | Integer | Double> ^<Alien>"with: arg9 "<Alien | Integer | Double> ^<Alien>"with: arg10 "<Alien | Integer | Double> ^<Alien>"
	
	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" floatParmsSignature: byteArray "<ByteArray | nil>"  with: firstArg "<Alien | Integer | Double> ^<Alien>"with: secondArg "<Alien | Integer | Double> ^<Alien>"with: thirdArg "<Alien | Integer | Double> ^<Alien>"with: fourthArg "<Alien | Integer | Double> ^<Alien>"with: fifthArg "<Alien | Integer | Double> ^<Alien>"with: sixthArg "<Alien | Integer | Double> ^<Alien>"with: arg7 "<Alien | Integer | Double> ^<Alien>"with: arg8 "<Alien | Integer | Double> ^<Alien>"with: arg9 "<Alien | Integer | Double> ^<Alien>"with: arg10 "<Alien | Integer | Double> ^<Alien>"with: arg11 "<Alien | Integer | Double> ^<Alien>"

	<primitive: 'primCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result floatParmsSignature: byteArray "<ByteArray | nil>"  withArguments: aCollection "<Alien | Integer | Double> ^<Alien>"  

	<primitive: 'primVarArgsCallOutIntegralReturnWithFloatParms' error: errorCode module: 'IA32ABI'>
	^ self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | SmallInteger> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer> ^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" with: firstArg "<Alien | Integer>" with: secondArg "<Alien | Integer>" with: thirdArg "<Alien | Integer>" with: fourthArg "<Alien | Integer>" with: fifthArg "<Alien | Integer>" with: sixthArg "<Alien | Integer>" with: arg7 "<Alien | Integer>" with: arg8 "<Alien | Integer>" with: arg9 "<Alien | Integer>" with: arg10 "<Alien | Integer>" with: arg11 "<Alien | Integer>" with: arg12 "<Alien | Integer>" "^<Alien>"
	<primitive: 'primCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-callouts' stamp: 'F Olivero 5/19/2009 01:05'!
primFFICallResult: result "<Alien>" withArguments: args "<Array of: Alien | Integer> ^<Alien>"
	<primitive: 'primVarArgsCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-library loading' stamp: 'F Olivero 5/18/2009 20:01'!
primFindSymbol: symbolName "<String> ^<Integer>"
	"The receiver is assumed to be a proxy whose first field contans the handle of a library
	 answered by primLoadLibrary.  Attempt to find symbolName in the library and answer
	 its address, or fail if the receiver is invalid or the symbol cannot be found."
	<primitive: 'primInLibraryFindSymbol' error: errorCode module: 'IA32ABI'>
	^errorCode == #'not found'
		ifTrue: [NotFoundError signal]
		ifFalse: [self primitiveFailed]! !

!Alien methodsFor: 'primitives-memory management' stamp: 'eem 12/21/2007 15:50'!
primFree
	"Free the memory referenced by the receiver, or fail if
	 the receiver is direct or the indirect pointer is invalid.
	 Note that if successful the address and size fields are
	 zero'ed, effectively preventing double-freeing hazards."
	<primitive: 'primBoxedFree' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/28/2007 17:53'!
replaceFrom: start "<Integer>" to: stop "<Integer>" with: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>"
	<primitive: 'primAlienReplace' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:21'!
signedByteAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedByteAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:22'!
signedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedByteAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'kdt 9/22/2009 11:31'!
signedIntAt: index
	^self signedLongAt: index! !

!Alien methodsFor: 'primitives-accessing' stamp: 'kdt 9/22/2009 11:32'!
signedIntAt: index put: value
	self signedLongAt: index put: value! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:21'!
signedLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedLongAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:18'!
signedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedLongAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 11/25/2008 21:04'!
signedLongLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedLongLongAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 11/25/2008 21:04'!
signedLongLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedLongLongAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/15/2007 17:49'!
signedShortAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedShortAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/15/2007 17:49'!
signedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedShortAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/8/2007 19:54'!
sizeField "^<Integer>"
	<primitive: 'primSizeField' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'F Olivero 2/26/2009 15:14'!
sizeFieldPut: size "<Integer> ^<Integer>"
	<primitive: 'primSizeFieldPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed ! !

!Alien methodsFor: 'accessing' stamp: 'F Olivero 5/18/2009 18:58'!
strcpy "^<ByteString>"
	^self strcpyFrom: 1! !

!Alien methodsFor: 'accessing' stamp: 'JohanBrichau 8/14/2009 14:43'!
strcpyFrom: startIndex "<Integer> ^<ByteString>"
	| length string |
	length := self strlenStartingAt: startIndex.
	string := ByteString new: length.
	self copyInto: string
		from: 1
		to: length 
		in: self
		startingAt: startIndex .
	^string! !

!Alien methodsFor: 'accessing' stamp: 'F Olivero 5/18/2009 18:37'!
strcpyUTF8 "^<String>"
	^self strcpyUTF8From: 1! !

!Alien methodsFor: 'accessing' stamp: 'John M McIntosh 11/23/2008 23:41'!
strcpyUTF8From: startIndex "<Integer> ^<String>"
	| length bytes |
	length := self strlenStartingAt: startIndex.
	self copyInto: (bytes := ByteArray new: length)
		from: 1 to: length
		in: self startingAt: startIndex.
	^bytes utf8ToIso! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/28/2007 17:59'!
strlenStartingAt: index "<Integer> ^<Integer>"
	<primitive: 'primStrlenFromStartIndex' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 1/8/2008 18:02'!
strlenThroughPointerAt: index "<Integer> ^<Integer>"
	<primitive: 'primStrlenThroughPointerAtIndex' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:20'!
unsignedByteAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedByteAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:19'!
unsignedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedByteAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'kdt 9/22/2009 11:33'!
unsignedIntAt: index 
	^ self unsignedLongAt: index! !

!Alien methodsFor: 'primitives-accessing' stamp: 'kdt 9/22/2009 11:34'!
unsignedIntAt: index put: value
	self unsignedLongAt: index put: value! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:18'!
unsignedLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/6/2007 15:16'!
unsignedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 11/25/2008 21:04'!
unsignedLongLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongLongAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'John M McIntosh 11/25/2008 21:04'!
unsignedLongLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongLongAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/15/2007 17:49'!
unsignedShortAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedShortAt' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!Alien methodsFor: 'primitives-accessing' stamp: 'eem 11/15/2007 17:49'!
unsignedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedShortAtPut' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

Alien variableByteSubclass: #FFICallbackReturnValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!FFICallbackReturnValue commentStamp: 'eem 4/14/2008 14:19' prior: 0!
An instance of FFICallbackReturnValue specifies a return value to be passed to a callback callee.  It is intended to have overlaid the following struct:
/*
 * Returning values from callbacks is done through a CallBackReturnSpec
 * which contains a type tag and values.  It is designed to be overlaid upon
 * an FFICallbackReturnProxy created at the Smalltalk level to return values.
 */
typedef struct {
    long type;
# define retint32  0 
# define retint64  1
# define retdouble 2
# define retstruct 3
    long _pad; /* so no doubt that valflt64 & valint32 et al are at byte 8 */
    union {
        long valint32;
        struct { long low, high; } valint64;
        double valflt64;
        struct { void *addr; long size; } valstruct;
    } rvs;
} CallBackReturnSpec;
!

!FFICallbackReturnValue class methodsFor: 'instance creation' stamp: 'eem 4/14/2008 14:18'!
dataSize
	"See class comment"
	^16! !

!FFICallbackReturnValue methodsFor: 'primitives-callbacks' stamp: 'eem 11/19/2007 10:55'!
primReturnFromContext: callbackContext "<ContextPart>" through: jmpbuf "<Integer>" "^not reached"
	"Return a value, as described by the receiver, to some C callee that has made a
	callback.   Reestablish the interpreter's state prior to the callback so that either
	the callout can return or subsequent callbacks can occur.  callbackContext is an
	activation of invokeCallback:stack:registers:jmpbuf:.  Its sender is the interpreter's
	state prior to the callback"
	<primitive: 'primReturnFromContextThrough' error: errorCode module: 'IA32ABI'>
	self primitiveFailed! !

!FFICallbackReturnValue methodsFor: 'instance initalization' stamp: 'eem 11/19/2007 16:27'!
returnDouble: f
	self signedLongAt: self typeOffset put: 2.
	self doubleAt: self valueOffset put: f asFloat! !

!FFICallbackReturnValue methodsFor: 'instance initalization' stamp: 'eem 11/19/2007 16:27'!
returnInteger: n
	self signedLongAt: self typeOffset put: 0.
	self signedLongAt: self valueOffset put: n asInteger! !

!FFICallbackReturnValue methodsFor: 'field offsets' stamp: 'eem 11/18/2007 13:45'!
typeOffset
	^1 "0"! !

!FFICallbackReturnValue methodsFor: 'field offsets' stamp: 'eem 4/14/2008 14:19'!
valueOffset
	^9 "8"! !

Alien variableByteSubclass: #FFICallbackThunk
	instanceVariableNames: ''
	classVariableNames: 'AccessProtect AllocatedThunks ExecutablePages'
	poolDictionaries: ''
	category: 'Alien-Core'!

!FFICallbackThunk commentStamp: 'eem 12/21/2007 15:53' prior: 0!
An instance of FFICallbackThunk is a reference to a machine-code thunk/trampoline that calls-back into the VM.  The reference can be passed to C code which can use it as a function pointer through which to call-back into Smalltalk.  The machine-code thunk/trampoline is different for each instance, hence its address is a unique key that can be used to assocuate the Smalltalk side of the call-back (e.g. a block) with the thunk.  Since thunks must be executable and some OSs may not provide default execute permission on memory returned by malloc we may not be able to use malloc directly.  Instead we rely on a primitive to provide memory that is guaranteed to be executable.  The FFICallbackThunk class>>allocateExectablePage primitive answers an Alien that references an executable piece of memory that is some (possiby unitary) multiple of the pagesize.  Class-side code then parcels out pieces of a page to individual thunks.  These pieces are recycled when thunks are reclaimed.  Since the first byte of a thunk is non-zero we can use it as a flag indicating if the piece is in use or not.

See Callback for the higher-level construct that represents a Smalltalk block to be run in response to a callback.  Callbacks wrap instances of FFICallbackThunk and arbitrary Alien instances that describe the stack layout for receiving arguments.

Class Variables
AccessProtect <Semaphore> critical section for ExecutablePages (de)allocation
AllocatedThunks <AlienWeakTable of <FFICallbackThunk -> Integer>> - weak collection of thunks, used to return thunk storage to the executable page pool.
ExecutablePages <Set of: Alien "executable page"> - collection of pages with execute permissions used to provide executable thunks!

!FFICallbackThunk class methodsFor: 'executable page management' stamp: 'eem 12/20/2007 17:14'!
allocateExecutablePage "^<Alien>"
	"Answer an Alien around a piece of (probably malloced) memory, of some multiple
	 of the pagesize, that has execute permissions set.  This memory can be parcelled
	 out to individual FFICallbackThunks to hold their thunk/trampoline machine code."
	<primitive: 'primAllocateExecutablePage' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 12/20/2007 17:06'!
dataSize
	^24 "see FFICallbackThunk initialize & initializeStdcall:"! !

!FFICallbackThunk class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:45'!
ensureNotifiedAtStartup
	"FFICallbackThunk needs to be notified at system startup to clear the ExecutablePages table."
	self ~~ FFICallbackThunk ifTrue: [^self].
	self ensureNotifiedAtStartupAfter: Alien! !

!FFICallbackThunk class methodsFor: 'finalization' stamp: 'eem 12/21/2007 15:17'!
finalize: address
	self freeBlockInExecutablePage: address! !

!FFICallbackThunk class methodsFor: 'executable page management' stamp: 'eem 12/20/2007 17:41'!
freeBlockInExecutablePage: address
	ExecutablePages do:
		[:alienPage| | alienAddress |
		alienAddress := alienPage address.
		(address >= alienAddress
		 and: [alienAddress + alienPage dataSize >= address]) ifTrue:
			[alienPage unsignedByteAt: address - alienAddress + 1 put: 0.
			 ^self]]
			! !

!FFICallbackThunk class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:47'!
initialize
	"FFICallbackThunk initialize"
	AccessProtect := Semaphore forMutualExclusion.
	AllocatedThunks := AlienWeakTable newForOwner: self.
	ExecutablePages := Set new.
	self ensureNotifiedAtStartup! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 15:32'!
newCCall
	"Answer a new instance for the C calling-convention."
	^self privateFinalizingNew initialize! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 15:32'!
newStdCall: argBytes
	"Answer a new instance for Windows' StdCall (Pascal) convention, given the
	 size of the arguments in bytes (bytes to cut-back from the stack on return)."
	^self privateFinalizingNew initializeStdcall: argBytes! !

!FFICallbackThunk class methodsFor: 'instance creation' stamp: 'eem 2/20/2008 10:48'!
privateFinalizingNew
	"Answer a new instance that will free its page when garbage collected."
	| address |
	address := self unusedBlockInExecutablePage.
	^AllocatedThunks
		add: (self atAddress: address)
		finalizing: address ! !

!FFICallbackThunk class methodsFor: 'class initialization' stamp: 'vb 5/8/2008 11:32'!
startUp: inANewImageSession
	"Any thunks in the finalization registry at the time the image comes up in a new session MUST NOT be finalized and should  immediately be discarded. Their thunk pointers are no longer valid."
	inANewImageSession ifTrue:
		[AllocatedThunks removeAll.
		ExecutablePages := Set new]! !

!FFICallbackThunk class methodsFor: 'executable page management' stamp: 'eem 12/21/2007 15:28'!
unusedBlockInExecutablePage
	AccessProtect critical:
		[ExecutablePages do:
			[:alien|
			1 to: alien dataSize - self dataSize by: self dataSize do:
				[:i|
				(alien unsignedByteAt: i) = 0 ifTrue:
					[alien unsignedByteAt: i put: 1.
					 ^alien address + i - 1]]]].
	^(AccessProtect critical: [ExecutablePages add: self allocateExecutablePage])
		unsignedByteAt: 1 put: 1;
		address! !

!FFICallbackThunk methodsFor: 'accessing' stamp: 'eem 1/29/2008 13:06'!
address
	"Thunks are hacked - again they need to be indirect objects with a 0 size field."
	^self addressField! !

!FFICallbackThunk methodsFor: 'memory management' stamp: 'eem 12/21/2007 15:38'!
free
	"Recycle the memory in the executable page containing the receiver's thunk.
	 Since the receiver is a derived pointer to a block in an executable page this
	 requires returning the block to the free pool."
	AllocatedThunks remove: self.
	AccessProtect critical:
		[| address |
		address := self addressField.
		address ~= 0 ifTrue:
			[self addressFieldPut: 0.
			 self class freeBlockInExecutablePage: address]]! !

!FFICallbackThunk methodsFor: 'instance initalization' stamp: 'eem 1/3/2008 17:14'!
initialize
	"Initialize the receiver with a __ccall thunk."
	"thunk:		push   %esp					0x54							0xa1905454
	 thunk+01:	push   %esp					0x54
	 thunk+02:	nop   						0x90
	 thunk+03:	mov    $thunkEntry,%eax	0xb8 0x00 0x00 0x00 0x00	0x00000000 - entry
	 thunk+08:	nop   						0x90							0x68909090
	 thunk+09:	nop   						0x90
	 thunk+10:	nop   						0x90
	 thunk+11:	push   $thunk				0x68 0x00 0x00 0x00 0x00	0x00000000 - thunk
	 thunk+16:	call   *%eax					0xff 0xd0						0xc483d0ff
	 thunk+18:	add    $0xC,%esp			0x83 0xc4 0x0C				0x9090c30C
	 thunk+21:	ret							0xc3
	 thunk+22:	nop							0x90
	 thunk+23:	nop							0x90"

	"Hack.  At first self is indirect so one can assign to the malloc'ed memory.
	Once that's done just set self to be direct, turning the reference into a pointer."
	self
		unsignedLongAt:  1 put: 16rB8905454;
		unsignedLongAt:  5 put: self primThunkEntryAddress;
		unsignedLongAt:  9 put: 16r68909090;
		unsignedLongAt: 13 put: self addressField;
		unsignedLongAt: 17 put: 16rC483D0FF;
		unsignedLongAt: 21 put: 16r9090C30C.
	self sizeFieldPut: 4! !

!FFICallbackThunk methodsFor: 'instance initalization' stamp: 'eem 1/3/2008 17:14'!
initializeStdcall: bytes
	"Initialize the receiver with a __stdcall thunk with bytes argument bytes."
	"thunk:		push   %esp					0x54							0xa1905454
	 thunk+01:	push   %esp					0x54
	 thunk+02:	nop   						0x90
	 thunk+03:	mov    $thunkEntry,%eax	0xb8 0x00 0x00 0x00 0x00	0x00000000 - entry
	 thunk+08:	nop   						0x90							0x68909090
	 thunk+09:	nop   						0x90
	 thunk+10:	nop   						0x90
	 thunk+11:	push   $thunk				0x68 0x00 0x00 0x00 0x00	0x00000000 - thunk
	 thunk+16:	call   *%eax					0xff 0xd0						0xc483d0ff
	 thunk+18:	add    $0xC,%esp			0x83 0xc4 0x0C				0xBYTSc20C
	 thunk+21:	ret    $bytes				0xc2 0xBY 0xTS"

	"Hack.  At first self is indirect so one can assign to the malloc'ed memory.
	Once that's done just set self to be direct, turning the reference into a pointer."
	self
		unsignedLongAt:  1 put: 16rB8905454;
		unsignedLongAt:  5 put: self primThunkEntryAddress;
		unsignedLongAt:  9 put: 16r68909090;
		unsignedLongAt: 13 put: self addressField;
		unsignedLongAt:  17 put: 16rC483D0FF;
		unsignedShortAt: 21 put: 16rC20C;
		unsignedShortAt: 23 put: bytes.
	self sizeFieldPut: 4! !

!FFICallbackThunk methodsFor: 'primitives-memory management' stamp: 'eem 12/21/2007 15:45'!
primFree
	self shouldNotImplement! !

!FFICallbackThunk methodsFor: 'private-primitives' stamp: 'eem 11/15/2007 17:45'!
primThunkEntryAddress "^<Integer>"
	"Answer the address of the entry-point for thunk callbacks:
		long thunkEntry(void *thunkp, long *stackp);
	 This is the function a callback thunk/trampoline should call to initiate a callback."
	<primitive: 'primThunkEntryAddress' error: errorCode module: 'IA32ABI'>
	^self primitiveFailed! !

!ByteArray methodsFor: '*Alien-Core' stamp: 'eem 1/25/2008 14:56'!
utf8ToIso
	| s i v1 v2 v3 v4 |
	s := WriteStream on: (String new: self size).
	i := 1.
	[i <= self size] whileTrue:
		[v1 := self at: i.
		 i := i + 1.
		 v1 < 128
			ifTrue: [ s nextPut: v1 asCharacter ]
			ifFalse:
				[(i > self size) ifTrue:
					[self error: 'illegal end-of-string, expected 2nd byte of UTF-8'].
				 v2 := self at: i.
				 i := i + 1.
				 v1 < 224
					ifTrue:
						[s nextPut: ((v2 bitAnd: 63) bitOr: ((v1 bitAnd: 31) bitShift: 6)) asCharacter]
					ifFalse:
						[(i > self size) ifTrue:
							[self error: 'illegal end-of-string, expected 3rd byte of UTF-8'].
						 v3 := self at: i.
						 i := i + 1.
						 v1 < 240
							ifTrue:
								[s nextPut: ((v3 bitAnd: 63) bitOr: (((v2 bitAnd: 63) bitShift: 6) bitOr: ((v1 bitAnd: 31) bitShift: 12))) asCharacter]
							ifFalse:
								[(i > self size) ifTrue:
							[self error: 'illegal end-of-string, expected 4th byte of UTF-8'].
							 v4 := self at: i.
							 i := i + 1.
							 v1 < 248
									ifTrue: [s nextPut: ((v4 bitAnd: 63) bitOr: (((v3 bitAnd: 63) bitShift: 6) bitOr: (((v2 bitAnd: 63) bitShift: 12) bitOr: ((v1 bitAnd: 15) bitShift: 18)))) asCharacter]
									ifFalse:
										[self error: 'illegal UTF-8 character']]]]].
	^s contents! !

!True methodsFor: '*Alien-Core' stamp: 'F Olivero 5/12/2009 18:00'!
asAlien

	^ 1
	! !

Object subclass: #AlienLibrary
	instanceVariableNames: 'alienHandle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

AlienLibrary class
	instanceVariableNames: 'uniqueInstance'!

AlienLibrary class
	instanceVariableNames: 'uniqueInstance'!

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:45'!
double: aString 
	^ self 
		double: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:45'!
double: aString with: arg1 
	^ self 
		double: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/22/2009 10:33'!
double: aString withArguments: anArray 
	| result |
	result := Alien new: Alien sizeofDouble.
	(self lookup: aString) 
		primFFICallDoubleResult: result
		withArguments: anArray.
	^ result doubleAt: 1! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:50'!
float: aString 
	^ self 
		float: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:50'!
float: aString with: arg1
	^ self 
		float: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/22/2009 10:34'!
float: aString withArguments: anArray 
	| result |
	result := Alien new: Alien sizeofFloat.
	(self lookup: aString) 
		primFFICallResult: result
		withArguments: anArray.
	^ result floatAt: 1! !

!AlienLibrary class methodsFor: 'as yet unclassified' stamp: 'F Olivero 5/19/2009 20:19'!
forgetUniqueInstance
	uniqueInstance := nil! !

!AlienLibrary class methodsFor: 'initialization' stamp: 'kdt 9/22/2009 11:43'!
initialize
	"self initialize"
	
	super initialize.
	Smalltalk removeFromShutDownList: self.
	Smalltalk addToStartUpList: self.
	! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:43'!
int: aString 
	^ self 
		int: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:43'!
int: aString with: arg1 
	^ self 
		int: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:44'!
int: aString with: arg1 with: arg2 
	^ self 
		int: aString
		withArguments: (Array 
				with: arg1
				with: arg2)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:44'!
int: aString with: arg1 with: arg2 with: arg3 
	^ self 
		int: aString
		withArguments: (Array 
				with: arg1
				with: arg2
				with: arg3)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:44'!
int: aString with: arg1 with: arg2 with: arg3 with: arg4 
	^ self 
		int: aString
		withArguments: (Array 
				with: arg1
				with: arg2
				with: arg3
				with: arg4)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:44'!
int: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 
	^ self 
		int: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array with: arg5)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:44'!
int: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 
	^ self 
		int: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array 
				with: arg5
				with: arg6)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:45'!
int: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 with: arg7 
	^ self 
		int: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array 
				with: arg5
				with: arg6
				with: arg7)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/22/2009 11:37'!
int: aString withArguments: anArray
	| result |
	result := Alien new: Alien sizeofInt.
	(self lookup: aString) 
		primFFICallResult: result
		withArguments: anArray.
	^ result asSignedInt! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
long: aString 
	^ self 
		long: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
long: aString with: arg1
	^ self 
		long: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/22/2009 11:37'!
long: aString withArguments: anArray
	| result |
	result := Alien new: Alien sizeofLong.
	(self lookup: aString) 
		primFFICallResult: result
		withArguments: anArray.
	^ result asSignedLong! !

!AlienLibrary class methodsFor: 'private' stamp: 'kdt 9/21/2009 16:18'!
lookup: aString 
	^ self uniqueInstance alienMethodNamed: aString! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:52'!
pointer: aString 
	^ self 
		pointer: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
pointer: aString with: arg1 
	^ self 
		pointer: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
pointer: aString with: arg1 with: arg2 
	^ self 
		pointer: aString
		withArguments: (Array 
				with: arg1
				with: arg2)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
pointer: aString with: arg1 with: arg2 with: arg3 
	^ self 
		pointer: aString
		withArguments: (Array 
				with: arg1
				with: arg2
				with: arg3)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
pointer: aString with: arg1 with: arg2 with: arg3 with: arg4 
	^ self 
		pointer: aString
		withArguments: (Array 
				with: arg1
				with: arg2
				with: arg3
				with: arg4)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
pointer: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 
	^ self 
		pointer: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array with: arg5)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:51'!
pointer: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 
	^ self 
		pointer: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array 
				with: arg5
				with: arg6)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:35'!
pointer: aString withArguments: anArray 
	| result |
	result := Alien forPointer: 0.
	(self lookup: aString) 
		primFFICallResult: result
		withArguments: anArray.
	^ result! !

!AlienLibrary class methodsFor: 'system startup' stamp: 'kdt 9/21/2009 16:16'!
startUp: resuming 
	resuming ifFalse: [ ^ self ].
	self forgetUniqueInstance.
	self allSubclasses do: [ :each | each forgetUniqueInstance ]! !

!AlienLibrary class methodsFor: 'as yet unclassified' stamp: 'F Olivero 5/18/2009 19:45'!
uniqueInstance
	uniqueInstance isNil ifTrue:[ uniqueInstance := self new ].
	^ uniqueInstance! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:49'!
unsignedInt: aString 
	^ self 
		unsignedInt: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:49'!
unsignedInt: aString with: arg1 
	^ self 
		unsignedInt: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/22/2009 11:37'!
unsignedInt: aString withArguments: anArray
	| result |
	result := Alien new: Alien sizeofInt.
	(self lookup: aString) 
		primFFICallResult: result
		withArguments: anArray.
	^ result asUnsignedInt! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:53'!
unsignedLong: aString 
	^ self 
		unsignedLong: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:53'!
unsignedLong: aString with: arg1 
	^ self 
		unsignedLong: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/22/2009 11:37'!
unsignedLong: aString withArguments: anArray
	| result |
	result := Alien new: Alien sizeofLong.
	(self lookup: aString) 
		primFFICallResult: result
		withArguments: anArray.
	^ result asUnsignedLong! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:52'!
void: aString 
	self 
		void: aString
		withArguments: Array new! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:52'!
void: aString with: arg1 
	self 
		void: aString
		withArguments: (Array with: arg1)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:52'!
void: aString with: arg1 with: arg2 
	self 
		void: aString
		withArguments: (Array 
				with: arg1
				with: arg2)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:52'!
void: aString with: arg1 with: arg2 with: arg3 
	self 
		void: aString
		withArguments: (Array 
				with: arg1
				with: arg2
				with: arg3)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:53'!
void: aString with: arg1 with: arg2 with: arg3 with: arg4 
	self 
		void: aString
		withArguments: (Array 
				with: arg1
				with: arg2
				with: arg3
				with: arg4)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:52'!
void: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 
	self 
		void: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array with: arg5)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:53'!
void: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 
	self 
		void: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array 
				with: arg5
				with: arg6)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:53'!
void: aString with: arg1 with: arg2 with: arg3 with: arg4 with: arg5 with: arg6 with: arg7 
	self 
		void: aString
		withArguments: (Array 
			with: arg1
			with: arg2
			with: arg3
			with: arg4) , (Array 
				with: arg5
				with: arg6
				with: arg7)! !

!AlienLibrary class methodsFor: 'callouts' stamp: 'kdt 9/21/2009 16:35'!
void: aString withArguments: anArray 
	(self lookup: aString) 
		primFFICallResult: nil
		withArguments: anArray! !

!AlienLibrary methodsFor: 'private' stamp: 'F Olivero 5/18/2009 20:48'!
alienMethodNamed: aString 
	| address |
	address := alienHandle primFindSymbol: aString.
	^ Alien forPointer: address.! !

!AlienLibrary methodsFor: 'alien functions' stamp: 'F Olivero 5/18/2009 17:50'!
ffiPrintString: anAlienCString
	| result alienMethod |
	result :=  Alien new: 4.
	alienMethod := self alienMethodNamed: 'ffiPrintString'.
	alienMethod
		primFFICallResult: result
		with: anAlienCString pointer.		
	^ result			
		
	! !

!AlienLibrary methodsFor: 'alien functions - tests' stamp: 'F Olivero 5/19/2009 01:37'!
ffiTestLongLong10a2: arguments
	| result alienMethod |
	result :=  Alien new: 8.
	alienMethod := self alienMethodNamed: 'ffiTestLongLong10a2'.
	alienMethod
		primFFICallResult: result
		with: ( arguments at:1 )
		with: ( arguments at:2 )
		with: ( arguments at:3 )
		with: ( arguments at:4 )
		with: ( arguments at:5 )
		with: ( arguments at:6 )
		with: ( arguments at:7 )
		with: ( arguments at:8 )
		with: ( arguments at:9 )
		with: ( arguments at:10 )
		with: ( arguments at:11)
		with: ( arguments at:12 ).
	^ result	
	! !

!AlienLibrary methodsFor: 'alien functions - tests' stamp: 'F Olivero 5/18/2009 17:40'!
ffiTestLongLong8: arguments
	| result alienMethod |
	result :=  Alien new: 8.
	alienMethod := self alienMethodNamed:'ffiTestLongLong8'.
	^ alienMethod primFFICallResult: result
		with: (arguments at:1)
		with: (arguments at:2)
		with: (arguments at:3)
		with: (arguments at:4)
		with: (arguments at:5)
		with: (arguments at:6)
		with: (arguments at:7)
		with: (arguments at:8)
		with: (arguments at:9)
		with: (arguments at:10).
	! !

!AlienLibrary methodsFor: 'alien functions - tests' stamp: 'F Olivero 5/18/2009 17:37'!
ffiTestLongLonga2: arguments
	| result alienMethod |
	result :=  Alien new: 8.
	alienMethod := self alienMethodNamed:'ffiTestLongLonga2'.
	^ alienMethod primFFICallResult: result
		with: (arguments at:1)
		with: (arguments at:2)
		with: (arguments at:3)
		with: (arguments at:4).

	! !

!AlienLibrary methodsFor: 'initialize-release' stamp: 'F Olivero 5/18/2009 20:49'!
initialize
	super initialize.
	self load! !

!AlienLibrary methodsFor: 'testing' stamp: 'F Olivero 5/18/2009 20:55'!
isLoaded
	^ alienHandle notNil and:[ SmalltalkImage current listLoadedModules anySatisfy:[:each| each beginsWith: self libraryName]].! !

!AlienLibrary methodsFor: 'accessing' stamp: 'F Olivero 5/18/2009 17:24'!
libraryName
	^ 'IA32ABI'! !

!AlienLibrary methodsFor: 'accessing' stamp: 'F Olivero 5/18/2009 20:46'!
load
	| address |
	alienHandle notNil ifTrue:[ ^ self ].
	address := self primLoadLibrary: self libraryName.
	alienHandle := Alien new: 4.
	alienHandle unsignedLongAt: 1 put: address.
! !

!AlienLibrary methodsFor: 'private' stamp: 'F Olivero 5/19/2009 01:51'!
primLoadLibrary: libraryName "<String> ^<Integer>"
	"Attempt to load the shared library/DLL named libraryName and
	 answer a handle which can be used in primFindSymbol:inLibrary:,
	 or fail if libraryName is invalid or libraryName cannot be found."
	<primitive: 'primLoadLibrary' module: 'IA32ABI'>
	! !

!AlienLibrary methodsFor: 'accessing' stamp: 'F Olivero 5/18/2009 12:30'!
reload
	self unload;load.! !

!AlienLibrary methodsFor: 'accessing' stamp: 'F Olivero 5/18/2009 20:45'!
unload
	SmalltalkImage current unloadModule: self libraryName.
	alienHandle := nil.! !

Object subclass: #AlienWeakTable
	instanceVariableNames: 'accessProtect firstUnusedIndex lastUsedIndex weakArray strongArray owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!AlienWeakTable commentStamp: '<historical>' prior: 0!
This class supports simple post-mortem finalization of values associated with gc'ed objects.  An object to be finalized is registered in the table together with another object called 'the tag'. The finalizable object is held onto by the table weakly, the tag object--strongly. A table is initialized with the owner object, which is the object that performs the actual finalization. Some time after a finalizable object is garbage-collected, the owner is sent the #finalize: message with the object's tag as the argument.

Instance Variables:
	accessProtect <Semaphore>  - A mutex protecting state
	firstUnusedIndex <Integer> - The lowest index in strongArray that is empty (an invariant)
	lastUsedIndex <Integer> - The highest index in strongArray that is not empty (an invariant)
	weakArray <WeakArray> - The array of objects whose death we're interested in.
	strongArray <Array> - The array of corresponding objects that wll be passed to the owner when their corresponding element in weakArray is garbage collected.
	owner <Object> - The object that is sent finalize: with the tag of an object that has been garbage-collected.!

!AlienWeakTable class methodsFor: 'instance creation' stamp: 'eem 12/21/2007 15:00'!
newForOwner: anOwner
	^self basicNew initializeWithOwner: anOwner! !

!AlienWeakTable methodsFor: 'adding' stamp: 'vb 5/7/2008 08:34'!
add: anObject finalizing: postMortemFinalizationTag
	"Register a new object in the table, with the associated tag. The object is held onto weakly, the tag--strongly. The tag will be passed to the owner as the argument of #finalize: some time after the object is garbage collected."

	| size start delta |
	accessProtect critical:
		[size := strongArray size.
		 start := firstUnusedIndex].
	start to: size do:
		[:i|
		(strongArray at: i) == nil ifTrue:
			[accessProtect critical:
				[weakArray at: i put: anObject.
				 strongArray at: i put: postMortemFinalizationTag.
				 firstUnusedIndex := i + 1.
				 i > lastUsedIndex ifTrue: [lastUsedIndex := i]].
			 ^anObject]].
	delta := size min: 4096.
	accessProtect critical:
		[(weakArray := weakArray copyGrownBy: delta)
			 at: size + 1 put: anObject.
		 (strongArray := strongArray  copyGrownBy: delta)
			at: size put: postMortemFinalizationTag.
		 firstUnusedIndex := size + 1.
		 lastUsedIndex := size].
	^anObject! !

!AlienWeakTable methodsFor: 'accessing' stamp: 'kdt 9/22/2009 12:05'!
allObjects
	| strongReferences |
	accessProtect critical: 
		[ strongReferences := Array new: lastUsedIndex.
		strongReferences 
			replaceFrom: 1
			to: lastUsedIndex
			with: weakArray
			startingAt: 1 ].
	^ strongReferences reject: [ :each | each isNil ]! !

!AlienWeakTable methodsFor: 'finalization' stamp: 'vb 5/7/2008 08:35'!
finalizeValues
	"Sent periodically by the finalization loop to check for any dead objects and report their death to the owner."
	
	accessProtect critical:
		[| usedIndex value newLastUsedIndex |
		 newLastUsedIndex := 0.
		 1 to: lastUsedIndex do:
			[:i|
			(value := strongArray at: i) ifNotNil:
				[(weakArray at: i)
					ifNil:
						[strongArray at: i put: nil.
						 owner finalize: value.
						 firstUnusedIndex > i ifTrue:
							[firstUnusedIndex := i]]
					ifNotNil: [newLastUsedIndex := i]]].
		 lastUsedIndex := newLastUsedIndex]! !

!AlienWeakTable methodsFor: 'initialize-release' stamp: 'vb 5/8/2008 11:32'!
initializeWithOwner: anObject
	accessProtect := Semaphore forMutualExclusion.
	firstUnusedIndex := 1.
	lastUsedIndex := 0.
	weakArray := WeakArray new: 1024.
	strongArray := Array new: 1024.
	owner := anObject.
	WeakArray addWeakDependent: self! !

!AlienWeakTable methodsFor: 'accessing' stamp: 'kdt 9/22/2009 11:51'!
objectWithTag: aTag
	^ accessProtect critical: 
		[ weakArray at: (strongArray 
				indexOf: aTag
				ifAbsent: [ ^ nil ]) ]! !

!AlienWeakTable methodsFor: 'removing' stamp: 'eem 1/11/2008 12:10'!
remove: anObject
	1 to: lastUsedIndex do:
		[:i|
		(weakArray at: i) == anObject ifTrue:
			[accessProtect critical:
				[strongArray at: i put: nil.
				 firstUnusedIndex > i ifTrue:
					[firstUnusedIndex := i].
				 i = lastUsedIndex ifTrue:
					[lastUsedIndex := i - 1]].
			 ^anObject]].
	^anObject! !

!AlienWeakTable methodsFor: 'removing' stamp: 'vb 5/8/2008 11:31'!
removeAll

	accessProtect critical:
		[1 to: lastUsedIndex do:
			[:index |
			strongArray at: index put: nil.
			weakArray at: index put: nil].
		firstUnusedIndex := 1.
		lastUsedIndex := 0]! !

!AlienWeakTable methodsFor: 'removing' stamp: 'kdt 9/22/2009 11:49'!
removeWithTag: aTag 
	| i |
	accessProtect critical: 
		[ i := strongArray 
			indexOf: aTag
			ifAbsent: [ ^ nil ].
		strongArray 
			at: i
			put: nil.
		weakArray 
			at: i
			put: nil.
		i = lastUsedIndex ifTrue: [ lastUsedIndex := i - 1 ] ]! !

Object subclass: #Callback
	instanceVariableNames: 'block thunk argsProxy resultProxy'
	classVariableNames: 'FlushThunkToCallbackMap ThunkToCallbackMap'
	poolDictionaries: ''
	category: 'Alien-Core'!

!Callback commentStamp: 'eem 11/26/2007 15:37' prior: 0!
Callbacks encapsulate callbacks from the outside world.  They allow Smalltalk blocks to be evaluated and answer their results to external (e.g. C) callees.

Instance Variables:
block <BlockContext> - The Smalltalk code to be run in response to external code invoking the callback.
thunk <FFICallbackThunk> - the wrapper around the machine-code thunk that initiates the callback and whose address should be passed to C
argsProxy <Alien> - the wrapper around the thunk's incomming stack pointer, used to extract arguments from the stack.
resultProxy <FFICalbackReturnValue> - the specification of the block's return value and its C type so that the result can be passed back in the right low-level form.

Class Variables:
ThunkToCallbackMap <Dictionary of: thunkAddress <Integer> -> callback <Callback>> - used to lookup the Callback associated with a specific thunk address on callback.  See FFICallbackThunk.!

!Callback class methodsFor: 'instance creation' stamp: 'eem 11/26/2007 15:38'!
block: aBlock "<BlockContext>" argsClass: argsProxyClass "<Alien class>" "^<Callback>"
	"Answer a Callback that evaluates aBlock and whose thunk should conform to the C calling convention."
	^self new
		block: aBlock
		thunk: FFICallbackThunk newCCall
		argsProxy: argsProxyClass newForCallbackArgs! !

!Callback class methodsFor: 'instance creation' stamp: 'eem 11/26/2007 15:38'!
block: aBlock "<BlockContext>" stdcallArgsClass: argsProxyClass "<Alien class>" "^<Callback>"
	"Answer a Callback that evaluates aBlock and whose thunk should conform to the
	 stdcall (Pascal) calling convention with a stack size of argsProxyClass dataSize."
	^self new
		block: aBlock
		thunk: (FFICallbackThunk newStdCall: argsProxyClass dataSize)
		argsProxy: argsProxyClass newForCallbackArgs! !

!Callback class methodsFor: 'evaluation' stamp: 'vb 3/26/2008 12:09'!
evaluateCallbackForThunk: thunkPtr "<Integer>" stack: stackPtr "<Integer>" "^<FFICallbackReturnValue>"
	| callback |
	callback := ThunkToCallbackMap
		at: thunkPtr
		ifAbsent: [^self error: 'could not locate Callback instance corresponding to thunk address'].
	callback isNil
		ifTrue: [self error: 'Callback instance for this thunk address has been garbage collected']
		ifFalse: [^callback valueWithStackPointer: stackPtr]! !

!Callback class methodsFor: 'finalization' stamp: 'eem 2/20/2008 16:11'!
finalizeValues
	FlushThunkToCallbackMap := true! !

!Callback class methodsFor: 'class initialization' stamp: 'eem 2/27/2008 16:34'!
initialize
	"Callback initialize"
	[self setup] "Monticello... grrr...."
		on: Error
		do: [:ex| ]! !

!Callback class methodsFor: 'class initialization' stamp: 'Ahe 2/26/2008 15:51'!
setup
	"Callback setup"
	| prevMap |
	FlushThunkToCallbackMap := false.
	(WeakArray hasWeakDependent: self) ifFalse:
		[WeakArray addWeakDependent: self].
	prevMap := ThunkToCallbackMap.
	ThunkToCallbackMap := WeakValueDictionary new.
	prevMap notNil ifTrue:
		[prevMap keysAndValuesDo:
			[:thunkAddress :callback|
			ThunkToCallbackMap at: thunkAddress put: callback]]! !

!Callback methodsFor: 'instance initialization' stamp: 'eem 2/20/2008 16:11'!
block: aBlock "<BlockContext>" thunk: thunkWrapper "<FFICallbackThunk>" argsProxy: argsWrapper "<Alien>"
	block := aBlock.
	thunk := thunkWrapper.
	argsProxy := argsWrapper.
	resultProxy := FFICallbackReturnValue new.
	FlushThunkToCallbackMap ifTrue:
		[FlushThunkToCallbackMap := false.
		 ThunkToCallbackMap keys do:
			[:k| (ThunkToCallbackMap at: k) ifNil: [ThunkToCallbackMap removeKey: k]]].
	ThunkToCallbackMap at: thunk addressField put: self! !

!Callback methodsFor: 'accessing' stamp: 'eem 11/19/2007 16:54'!
thunk
	^thunk! !

!Callback methodsFor: 'evaluation' stamp: 'vb 12/5/2007 09:06'!
valueWithStackPointer: stackPtr "<Integer>" "^<FFICallbackReturnValue>"
	argsProxy addressFieldPut: stackPtr.
	"Cloning the block to allow reentrant callbacks in Squeak."
	block clone value: argsProxy value: resultProxy.
	^resultProxy! !

Object subclass: #UnsafeAlien
	instanceVariableNames: 'nonPointerObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!UnsafeAlien commentStamp: 'John M McIntosh 12/15/2008 08:51' prior: 0!
Instances of UnsafeAlien represent the addresses of heap-resident non-pointer Smalltalk objects as actual parameters in FFI call-outs.  An UnsafeAlien on (e.g.) a ByteString used as a parameter in an FFI call causes the FFI machinery to pass the address of the first byte in the ByteString.  THIS IS UNSAFE!!  It is unsafe because
a) the garbage collector can potentially move the ByteString (or any other object) during the call, because the call may call-back, invoking the garbage collector,
b) if external code retains the address for longer than the duration of the call and dereferences it in a subsequent call the object may have moved in the mean time,
c) the address of the object is passed without any other potentially necessary conversions such as null-termination
d) the hundred other problems this benighted author hasn't thought of.
Hence UnsafeAlien is to be used carefully by clients that know that the usage is safe.
You have been warned ;)

Create instances via
	UnsafeAlien forPointerTo: 'You are on your own!!', (ByteString with: (Character value: 0))
	
Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0
!

!UnsafeAlien class methodsFor: 'instance creation' stamp: 'John M McIntosh 12/15/2008 08:55'!
forPointerTo: anObject
	"UnsafeAlien forPointerTo: 'You are on your own!!', (ByteString with: (Character value: 0))"
	
(anObject class instSpec between: 6 and: 8) ifFalse:
		[^self error: 'only use UnsafeAlien on instances of isBits classes (other than CompiledMethod)!!'].
	^self new nonPointerObject: anObject! !

!UnsafeAlien class methodsFor: 'class initialization' stamp: 'F Olivero 5/19/2009 01:32'!
initialization
"UnsafeAlien must be in the specialObjectsArray to enable the Newsqueak FFI.
	 Because of the bootstrap the specialObjectsArray may already have been
	 partially initialized with the callback selector and to be large enough (as part of
	 loading immutability), but it will be missing UnsafeAlien until UnsafeAlien is loaded.
	 So check if the specialObjectsArray is of the expected size before recreating
	 and slam UnsafeAlien in if the specialObjectsArray is already large enough."

	| index |
	index := self specialObjectsArrayIndex.
	((Smalltalk includesKey: #ObjectMemory)
	 and: [((Smalltalk at: #ObjectMemory) classPool at: #ClassUnsafeAlien ifAbsent: []) ~~ (index - 1)]) ifTrue:
		[self error: 'index probably wrong'].

	Smalltalk specialObjectsArray size < index ifTrue:
		[Smalltalk recreateSpecialObjectsArray].
	
	(Smalltalk specialObjectsArray size < index
	 or: [(Smalltalk specialObjectsArray at: index) ~~ nil
		and: [(Smalltalk specialObjectsArray at: index) ~~ self]]) ifTrue:
		[self error: 'SystemDictionary>>recreateSpecialObjectsArray appears incompatible'].

	(Smalltalk specialObjectsArray at: index) == nil ifTrue:
		[Smalltalk specialObjectsArray at: index put: self]! !

!UnsafeAlien class methodsFor: 'class initialization' stamp: 'F Olivero 5/19/2009 01:33'!
specialObjectsArrayIndex
	^ 55! !

!UnsafeAlien methodsFor: 'accessing' stamp: 'John M McIntosh 12/15/2008 08:57'!
nonPointerObject
	^ nonPointerObject! !

!UnsafeAlien methodsFor: 'accessing' stamp: 'John M McIntosh 12/15/2008 08:57'!
nonPointerObject: anObject
	nonPointerObject := anObject! !

!Decompiler methodsFor: '*Alien-Core-override' stamp: 'eem 7/13/2007 17:09'!
decompile: aSelector in: aClass method: aMethod using: aConstructor

	| block node |
	constructor := aConstructor.
	method := aMethod.
	self initSymbols: aClass.  "create symbol tables"
	method isQuick
		ifTrue: [block := self quickMethod]
		ifFalse: 
			[stack := OrderedCollection new: method frameSize.
			caseExits := OrderedCollection new.
			statements := OrderedCollection new: 20.
			super method: method pc: method initialPC.
			"skip primitive error code store if necessary"
			(method primitive ~= 0 and: [self willStore]) ifTrue:
				[pc := pc + 2.
				 tempVars := tempVars asOrderedCollection].
			block := self blockTo: method endPC + 1.
			stack isEmpty ifFalse: [self error: 'stack not empty']].
	node := constructor
		codeMethod: aSelector
		block: block
		tempVars: tempVars
		primitive: method primitive
		class: aClass.
	method primitive > 0 ifTrue:
		[node removeAndRenameLastTempIfErrorCode].
	^node! !

!SystemDictionary methodsFor: '*Alien-Core-override' stamp: 'F Olivero 5/18/2009 22:22'!
recreateSpecialObjectsArray
	"Smalltalk recreateSpecialObjectsArray"
	"The Special Objects Array is an array of object pointers used
	by the
	Squeak virtual machine. Its contents are critical and
	unchecked, so don't even think of playing here unless you
	know what you are doing."
	| newArray |
	newArray := Array new: 55.
	"Nil false and true get used throughout the interpreter"
	newArray at: 1 put: nil.
	newArray at: 2 put: false.
	newArray at: 3 put: true.
	"This association holds the active process (a ProcessScheduler)"
	newArray at: 4 put: (self associationAt: #Processor).
	"Numerous classes below used for type checking and instantiation"
	newArray at: 5 put: Bitmap.
	newArray at: 6 put: SmallInteger.
	newArray at: 7 put: ByteString.
	newArray at: 8 put: Array.
	newArray at: 9 put: Smalltalk.
	newArray at: 10 put: Float.
	newArray at: 11 put: MethodContext.
	newArray at: 12 put: BlockContext.
	newArray at: 13 put: Point.
	newArray at: 14 put: LargePositiveInteger.
	newArray at: 15 put: Display.
	newArray at: 16 put: Message.
	newArray at: 17 put: CompiledMethod.
	newArray at: 18 put: (self specialObjectsArray at: 18).
	"(low space Semaphore)"
	newArray at: 19 put: Semaphore.
	newArray at: 20 put: Character.
	newArray at: 21 put: #doesNotUnderstand:.
	newArray at: 22 put: #cannotReturn:.
	newArray at: 23 put: nil.
	"An array of the 32 selectors that are compiled as special bytecodes,
	 paired alternately with the number of arguments each takes."
	newArray at: 24 put: #(	#+ 1 #- 1 #< 1 #> 1 #<= 1 #>= 1 #= 1 #~= 1
							#* 1 #/ 1 #\\ 1 #@ 1 #bitShift: 1 #// 1 #bitAnd: 1 #bitOr: 1
							#at: 1 #at:put: 2 #size 0 #next 0 #nextPut: 1 #atEnd 0 #== 1 #class 0
							#blockCopy: 1 #value 0 #value: 1 #do: 1 #new 0 #new: 1 #x 0 #y 0 ).
	"An array of the 255 Characters in ascii order."
	newArray at: 25 put: ((0 to: 255) collect: [:ascii | Character value: ascii]).
	newArray at: 26 put: #mustBeBoolean.
	newArray at: 27 put: ByteArray.
	newArray at: 28 put: Process.
	"An array of up to 31 classes whose instances will have compact headers"
	newArray at: 29 put: self compactClassesArray.
	newArray at: 30 put: (self specialObjectsArray at: 30).
	"(delay Semaphore)"
	newArray at: 31 put: (self specialObjectsArray at: 31).
	"(user interrupt Semaphore)"
	"Prototype instances that can be copied for fast initialization"
	newArray at: 32 put: (Float new: 2).
	newArray at: 33 put: (LargePositiveInteger new: 4).
	newArray at: 34 put: Point new.
	newArray at: 35 put: #cannotInterpret:.
	"Note: This must be fixed once we start using context prototypes (yeah, right)"
	"(MethodContext new: CompiledMethod fullFrameSize)."
	newArray at: 36 put: (self specialObjectsArray at: 36). "Is the prototype MethodContext (unused by the VM)"
	newArray at: 37 put: BlockClosure.
	"(BlockContext new: CompiledMethod fullFrameSize)."
	newArray at: 38 put: (self specialObjectsArray at: 38). "Is the prototype BlockContext (unused by the VM)"
	newArray at: 39 put: (self specialObjectsArray at: 39).	"preserve external semaphores"
	"array of objects referred to by external code"
	newArray at: 40 put: PseudoContext.
	newArray at: 41 put: nil.
	"finalization Semaphore"
	newArray at: 42 put: ((self specialObjectsArray at: 42) ifNil: [Semaphore new]).
	newArray at: 43 put: LargeNegativeInteger.
	"External objects for callout.
	 Note: Written so that one can actually completely remove the FFI."
	newArray at: 44 put: (self at: #ExternalAddress ifAbsent: []).
	newArray at: 45 put: (self at: #ExternalStructure ifAbsent: []).
	newArray at: 46 put: (self at: #ExternalData ifAbsent: []).
	newArray at: 47 put: (self at: #ExternalFunction ifAbsent: []).
	newArray at: 48 put: (self at: #ExternalLibrary ifAbsent: []).
	newArray at: 49 put: #aboutToReturn:through:.
	newArray at: 50 put: #run:with:in:.
	newArray at: 51 put: nil.
	newArray at: 52 put: nil.
	newArray at: 53 put: (self at: #Alien ifAbsent: []).
	newArray at: 54 put: #invokeCallback:stack:registers:jmpbuf:.
	newArray at: 55 put: (self at: #UnsafeAlien ifAbsent: []).

	"Now replace the interpreter's reference in one atomic
	operation"
	self specialObjectsArray become: newArray! !

!UndefinedObject methodsFor: '*Alien-Core' stamp: 'eem 1/25/2008 13:10'!
asAlien
	^0! !

Error subclass: #NotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alien-Core'!

!NotFoundError methodsFor: 'initialize-release' stamp: 'mha 9/3/2009 16:28'!
initialize
	super initialize.
	self messageText: 'no messages'! !
Alien initialize!
FFICallbackThunk initialize!
AlienLibrary initialize!
Callback initialize!
